@@ -46,9 +46,7 @@ where
     type Map = Self;
     type Struct = Self;
     type TupleStruct = Self;
-    type StructVariant = Self;
-    type TupleVariant = Self;
-    type UnitVariant = Self;
+    type Variant = Self;
 
     #[inline]
     fn expecting(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
@@ -72,7 +70,7 @@ where
 
     #[inline]
     fn encode_bytes(mut self, bytes: &[u8]) -> Result<Self::Ok, Self::Error> {
-        L::encode_usize(&mut self.writer, bytes.len())?;
+        L::encode_usize(self.writer.borrow_mut(), bytes.len())?;
         self.writer.write_bytes(bytes)?;
         Ok(())
     }
@@ -80,7 +78,7 @@ where
     #[inline]
     fn encode_bytes_vectored(mut self, vectors: &[&[u8]]) -> Result<Self::Ok, Self::Error> {
         let len = vectors.into_iter().map(|v| v.len()).sum();
-        L::encode_usize(&mut self.writer, len)?;
+        L::encode_usize(self.writer.borrow_mut(), len)?;
 
         for bytes in vectors {
             self.writer.write_bytes(bytes)?;
@@ -91,14 +89,14 @@ where
 
     #[inline]
     fn encode_string(mut self, string: &str) -> Result<Self::Ok, Self::Error> {
-        L::encode_usize(&mut self.writer, string.len())?;
+        L::encode_usize(self.writer.borrow_mut(), string.len())?;
         self.writer.write_bytes(string.as_bytes())?;
         Ok(())
     }
 
     #[inline]
     fn encode_usize(mut self, value: usize) -> Result<Self::Ok, Self::Error> {
-        L::encode_usize(&mut self.writer, value)
+        L::encode_usize(self.writer.borrow_mut(), value)
     }
 
     #[inline]
@@ -123,22 +121,22 @@ where
 
     #[inline]
     fn encode_u16(mut self, value: u16) -> Result<Self::Ok, Self::Error> {
-        I::encode_unsigned(&mut self.writer, value)
+        I::encode_unsigned(self.writer.borrow_mut(), value)
     }
 
     #[inline]
     fn encode_u32(mut self, value: u32) -> Result<Self::Ok, Self::Error> {
-        I::encode_unsigned(&mut self.writer, value)
+        I::encode_unsigned(self.writer.borrow_mut(), value)
     }
 
     #[inline]
     fn encode_u64(mut self, value: u64) -> Result<Self::Ok, Self::Error> {
-        I::encode_unsigned(&mut self.writer, value)
+        I::encode_unsigned(self.writer.borrow_mut(), value)
     }
 
     #[inline]
     fn encode_u128(mut self, value: u128) -> Result<Self::Ok, Self::Error> {
-        I::encode_unsigned(&mut self.writer, value)
+        I::encode_unsigned(self.writer.borrow_mut(), value)
     }
 
     #[inline]
@@ -148,22 +146,22 @@ where
 
     #[inline]
     fn encode_i16(mut self, value: i16) -> Result<Self::Ok, Self::Error> {
-        I::encode_signed(&mut self.writer, value)
+        I::encode_signed(self.writer.borrow_mut(), value)
     }
 
     #[inline]
     fn encode_i32(mut self, value: i32) -> Result<Self::Ok, Self::Error> {
-        I::encode_signed(&mut self.writer, value)
+        I::encode_signed(self.writer.borrow_mut(), value)
     }
 
     #[inline]
     fn encode_i64(mut self, value: i64) -> Result<Self::Ok, Self::Error> {
-        I::encode_signed(&mut self.writer, value)
+        I::encode_signed(self.writer.borrow_mut(), value)
     }
 
     #[inline]
     fn encode_i128(mut self, value: i128) -> Result<Self::Ok, Self::Error> {
-        I::encode_signed(&mut self.writer, value)
+        I::encode_signed(self.writer.borrow_mut(), value)
     }
 
     #[inline]
@@ -190,7 +188,7 @@ where
 
     #[inline]
     fn encode_sequence(mut self, len: usize) -> Result<Self::Sequence, Self::Error> {
-        L::encode_usize(&mut self.writer, len)?;
+        L::encode_usize(self.writer.borrow_mut(), len)?;
         Ok(self)
     }
 
@@ -202,40 +200,30 @@ where
 
     #[inline]
     fn encode_map(mut self, len: usize) -> Result<Self::Map, Self::Error> {
-        L::encode_usize(&mut self.writer, len)?;
+        L::encode_usize(self.writer.borrow_mut(), len)?;
         Ok(self)
     }
 
     #[inline]
     fn encode_struct(mut self, fields: usize) -> Result<Self::Struct, Self::Error> {
-        L::encode_usize(&mut self.writer, fields)?;
+        L::encode_usize(self.writer.borrow_mut(), fields)?;
         Ok(self)
     }
 
     #[inline]
     fn encode_tuple_struct(mut self, len: usize) -> Result<Self::TupleStruct, Self::Error> {
-        L::encode_usize(&mut self.writer, len)?;
+        L::encode_usize(self.writer.borrow_mut(), len)?;
         Ok(self)
     }
 
     #[inline]
     fn encode_unit_struct(mut self) -> Result<Self::Ok, Self::Error> {
-        L::encode_usize(&mut self.writer, 0)?;
+        L::encode_usize(self.writer.borrow_mut(), 0)?;
         Ok(())
     }
 
     #[inline]
-    fn encode_struct_variant(self, _: usize) -> Result<Self::StructVariant, Self::Error> {
-        Ok(self)
-    }
-
-    #[inline]
-    fn encode_tuple_variant(self, _: usize) -> Result<Self::TupleVariant, Self::Error> {
-        Ok(self)
-    }
-
-    #[inline]
-    fn encode_unit_variant(self) -> Result<Self::UnitVariant, Self::Error> {
+    fn encode_variant(self) -> Result<Self::Variant, Self::Error> {
         Ok(self)
     }
 }
@@ -248,11 +236,11 @@ where
 {
     type Ok = ();
     type Error = W::Error;
-    type Encoder<'this> = StorageEncoder<&'this mut W, I, L> where Self: 'this;
+    type Encoder<'this> = StorageEncoder<W::Mut<'this>, I, L> where Self: 'this;
 
     #[inline]
     fn next(&mut self) -> Result<Self::Encoder<'_>, Self::Error> {
-        Ok(StorageEncoder::new(&mut self.writer))
+        Ok(StorageEncoder::new(self.writer.borrow_mut()))
     }
 
     #[inline]
@@ -269,11 +257,11 @@ where
 {
     type Ok = ();
     type Error = W::Error;
-    type Encoder<'this> = StorageEncoder<&'this mut W, I, L> where Self: 'this;
+    type Encoder<'this> = StorageEncoder<W::Mut<'this>, I, L> where Self: 'this;
 
     #[inline]
     fn next(&mut self) -> Result<Self::Encoder<'_>, Self::Error> {
-        Ok(StorageEncoder::new(&mut self.writer))
+        Ok(StorageEncoder::new(self.writer.borrow_mut()))
     }
 
     #[inline]
@@ -290,17 +278,17 @@ where
 {
     type Ok = ();
     type Error = W::Error;
-    type First<'this> = StorageEncoder<&'this mut W, I, L> where Self: 'this;
-    type Second<'this> = StorageEncoder<&'this mut W, I, L> where Self: 'this;
+    type First<'this> = StorageEncoder<W::Mut<'this>, I, L> where Self: 'this;
+    type Second<'this> = StorageEncoder<W::Mut<'this>, I, L> where Self: 'this;
 
     #[inline]
     fn first(&mut self) -> Result<Self::First<'_>, Self::Error> {
-        Ok(StorageEncoder::new(&mut self.writer))
+        Ok(StorageEncoder::new(self.writer.borrow_mut()))
     }
 
     #[inline]
     fn second(&mut self) -> Result<Self::Second<'_>, Self::Error> {
-        Ok(StorageEncoder::new(&mut self.writer))
+        Ok(StorageEncoder::new(self.writer.borrow_mut()))
     }
 
     #[inline]
