@@ -1,7 +1,7 @@
 //! Helpers for writing tests.
 
 use core::fmt::Debug;
-use musli::{Decode, Encode};
+use musli::{Decode, DefaultMode, Encode};
 
 /// Roundtrip encode the given value.
 #[macro_export]
@@ -29,8 +29,8 @@ macro_rules! rt {
 #[inline(never)]
 pub fn transcode<T, O>(value: T) -> O
 where
-    T: Debug + PartialEq + Encode,
-    O: for<'de> Decode<'de>,
+    T: Debug + PartialEq + Encode<DefaultMode>,
+    O: for<'de> Decode<'de, DefaultMode>,
 {
     let out = crate::to_vec(&value).expect("encode failed");
     let mut buf = &out[..];
